@@ -1,5 +1,6 @@
 """Test the pypdf._reader module."""
 import io
+import sys
 import time
 from io import BytesIO
 from pathlib import Path
@@ -17,6 +18,7 @@ from pypdf.errors import (
     DeprecationError,
     EmptyFileError,
     FileNotDecryptedError,
+    LimitReachedError,
     PdfReadError,
     PdfStreamError,
     WrongPasswordError,
@@ -30,13 +32,9 @@ from pypdf.generic import (
     TextStringObject,
 )
 
-from . import get_data_from_url, normalize_warnings
+from . import RESOURCE_ROOT, SAMPLE_ROOT, get_data_from_url, normalize_warnings
 
 HAS_AES = crypt_provider[0] in ["pycryptodome", "cryptography"]
-TESTS_ROOT = Path(__file__).parent.resolve()
-PROJECT_ROOT = TESTS_ROOT.parent
-RESOURCE_ROOT = PROJECT_ROOT / "resources"
-SAMPLE_ROOT = PROJECT_ROOT / "sample-files"
 
 
 NestedList = Union[int, None, list["NestedList"]]
@@ -1889,3 +1887,90 @@ def test_read_standard_xref_table__two_whitespace_characters_between_offset_and_
     reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
     assert len(reader.pages) == 1
     assert reader.pages[0].extract_text() == "Hello World!"
+
+
+@pytest.mark.enable_socket
+def test_root_object_recovery_limit(caplog):
+    url = "https://github.com/user-attachments/files/24525509/root_object_recovery_limit.pdf"
+    name = "root_object_recovery_limit.pdf"
+    data = get_data_from_url(url, name=name)
+
+    # Default limit.
+    reader = PdfReader(BytesIO(data))
+    with pytest.raises(
+            expected_exception=LimitReachedError, match=r"^Maximum Root object recovery limit reached\.$"
+    ):
+        _ = list(reader.pages)
+    message_numbers = {
+        int(message.split(" ", maxsplit=2)[1])
+        for message in caplog.messages
+        if message.startswith("Object ") and message.endswith(" 0 not defined.")
+    }
+    assert sorted(message_numbers) == list(range(5, 10001))
+
+    # Custom limit.
+    caplog.clear()
+    reader = PdfReader(BytesIO(data), root_object_recovery_limit=42)
+    with pytest.raises(
+            expected_exception=LimitReachedError, match=r"^Maximum Root object recovery limit reached\.$"
+    ):
+        _ = list(reader.pages)
+    message_numbers = {
+        int(message.split(" ", maxsplit=2)[1])
+        for message in caplog.messages
+        if message.startswith("Object ") and message.endswith(" 0 not defined.")
+    }
+    assert sorted(message_numbers) == list(range(5, 43))
+
+    # No limit. Do not run actual process for speed reasons.
+    reader = PdfReader(BytesIO(data), root_object_recovery_limit=None)
+    assert reader._root_object_recovery_limit == sys.maxsize
+
+    # Strict mode.
+    with pytest.raises(expected_exception=PdfReadError, match=r"^Broken xref table$"):
+        reader = PdfReader(BytesIO(data), strict=True)
+        _ = list(reader.pages)
+
+
+@pytest.mark.timeout(10)
+def test_rebuild_xref_table__speed():
+    total_len = 2_000_790
+    middle = b"\nstartxref   1\n % "
+    leading_len = 0x55E  # 1374
+    leading = b" " * leading_len
+    trailing = b" " * (total_len - leading_len - len(middle))
+    data = leading + middle + trailing
+
+    reader = PdfReader(BytesIO(data))
+    with pytest.raises(expected_exception=PdfReadError, match=r"^Cannot find Root object in pdf$"):
+        _ = list(reader.pages)
+
+
+def test_find_pdf_objects():
+    data = (
+        b"     \n"
+        b"  11 0 obj\n"
+        b"  12 0 obj\n"
+        b"13  1  obj\n"
+        b"ob\n"
+        b"ab obj\n"
+        b"42 1337 obj \n"
+        b"\n"
+    )
+
+    result = list(PdfReader._find_pdf_objects(data))
+    assert result == [(11, 0, 8), (12, 0, 19), (13, 1, 28), (42, 1337, 49)]
+
+
+@pytest.mark.parametrize(
+    ("data", "expected"),
+    [
+        (b"\n\ntrailer", []),
+        (b"\n\ntrailer abc", []),
+        (b"\n\ntrailer <<", [10]),
+        (b"\n\ntrailer << /Key null >>\n\n  trailer << /Key 42 >>\n", [10, 37])
+    ]
+)
+def test_find_pdf_trailers(data: bytes, expected: list[int]):
+    result = list(PdfReader._find_pdf_trailers(data))
+    assert result == expected
